@@ -27,7 +27,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "../xmscene/BasicSceneStructs.h"
 #include "BasicStructures.h"
 
-#define XM_NET_PROTOCOL_VERSION 4
+#define XM_NET_PROTOCOL_VERSION 6
 /*
 DELTA 1->2:
 clientInfos : add xmversion string
@@ -35,11 +35,15 @@ DELTA 2->3:
 add NA_udpBindValidation
 DELTA 3->4:
 add chatMessagePP (private/public)
+DELTA 4->5
+add slaveClientsPoints
+DELTA 5->6
+add pings
 */
 
-#define NETACTION_MAX_PACKET_SIZE 1024 * 2 // bytes
+#define NETACTION_MAX_PACKET_SIZE 1024 * 8 // bytes
 #define NETACTION_MAX_SUBSRC 4 // maximum 4 players by client
-#define XM_NET_MAX_EVENTS_SHOT_SIZE 1024
+#define XM_NET_MAX_EVENTS_SHOT_SIZE 1024 * 8
 
 class NetClient;
 class ServerThread;
@@ -59,6 +63,7 @@ enum NetActionType {
   TNA_clientsNumberQuery,
   TNA_playingLevel,
   TNA_changeClients,
+  TNA_slaveClientsPoints,
   TNA_playerControl,
   TNA_clientMode,
   TNA_prepareToPlay,
@@ -66,7 +71,8 @@ enum NetActionType {
   TNA_killAlert,
   TNA_gameEvents,
   TNA_srvCmd,
-  TNA_srvCmdAsw
+  TNA_srvCmdAsw,
+  TNA_ping
 };
 
 struct NetInfosClient {
@@ -74,6 +80,13 @@ struct NetInfosClient {
   std::string Name;
 };
 
+struct NetPointsClient {
+  int NetId;
+  int Points;
+};
+
+struct NetActionU;
+
 class NetAction {
   public:
   NetAction(bool i_forceTcp);
@@ -87,9 +100,18 @@ class NetAction {
   int getSource() const;
   int getSubSource() const;
 
-  static NetAction* newNetAction(void* data, unsigned int len);
+  static void getNetAction(NetActionU* o_netAction, void* data, unsigned int len);
+
   static void logStats();
-  static std::string getStats();
+
+  /* stats */
+  static unsigned int m_biggestTCPPacketSent;
+  static unsigned int m_biggestUDPPacketSent;
+  static unsigned int m_nbTCPPacketsSent;
+  static unsigned int m_nbUDPPacketsSent;
+  static unsigned int m_TCPPacketsSizeSent;
+  static unsigned int m_UDPPacketsSizeSent;
+  /* ***** */
 
   protected:
   void send(TCPsocket* i_tcpsd, UDPsocket* i_udpsd, UDPpacket* i_sendPacket, IPaddress* i_udpRemoteIP, const void* subPacketData, int subPacketLen);
@@ -107,23 +129,18 @@ class NetAction {
   // the server transfering a packet from a client x to others clients
   // => (x, [0,1,2,3])
 
-  static std::string getLine(void* data, unsigned int len, unsigned int* o_local_offset);
+  static std::string getLine         (void* data, unsigned int len, unsigned int* o_local_offset);
+  static std::string getLineCheckUTF8(void* data, unsigned int len, unsigned int* o_local_offset);
 
   private:
   static char m_buffer[NETACTION_MAX_PACKET_SIZE];
-  static unsigned int m_biggestTCPPacketSent;
-  static unsigned int m_biggestUDPPacketSent;
-  static unsigned int m_nbTCPPacketsSent;
-  static unsigned int m_nbUDPPacketsSent;
-  static unsigned int m_TCPPacketsSizeSent;
-  static unsigned int m_UDPPacketsSizeSent;
 
   bool m_forceTCP; // by default, xmoto try to use UDP when available ; for some actions, TCP can be forced
 };
 
 class NA_udpBind : public NetAction {
   public:
-  NA_udpBind(const std::string& i_key);
+  NA_udpBind(const std::string& i_key = "");
   NA_udpBind(void* data, unsigned int len);
   virtual ~NA_udpBind();
   std::string actionKey()    { return ActionKey; }
@@ -181,7 +198,7 @@ SRV : <udpBindValidation (ok, i can *send* udp to the client)
 
 class NA_clientInfos : public NetAction {
   public:
-  NA_clientInfos(int i_protocolVersion, const std::string& i_udpBindKey);
+  NA_clientInfos(int i_protocolVersion = -1, const std::string& i_udpBindKey = "");
   NA_clientInfos(void* data, unsigned int len);
   virtual ~NA_clientInfos();
   std::string actionKey()    { return ActionKey; }
@@ -204,6 +221,7 @@ class NA_clientInfos : public NetAction {
 /* replace chatMessage ; it allows public and private messages */
 class NA_chatMessagePP : public NetAction {
   public:
+  NA_chatMessagePP() : NetAction(true) {} // for preallocation
   NA_chatMessagePP(const std::string& i_msg, const std::string &i_me /* if empty, change nothing */, const std::vector<int>& i_private_people /* empty vector for public message -- require XM_NET_PROTOCOL_VERSION */);
   NA_chatMessagePP(void* data, unsigned int len);
   virtual ~NA_chatMessagePP();
@@ -226,7 +244,7 @@ class NA_chatMessagePP : public NetAction {
 
 class NA_chatMessage : public NetAction {
   public:
-  NA_chatMessage(const std::string& i_msg, const std::string &i_me);
+  NA_chatMessage(const std::string& i_msg = "", const std::string &i_me = "");
   NA_chatMessage(void* data, unsigned int len);
   virtual ~NA_chatMessage();
   std::string actionKey()    { return ActionKey; }
@@ -246,7 +264,7 @@ class NA_chatMessage : public NetAction {
 
 class NA_serverError : public NetAction {
  public:
-  NA_serverError(const std::string& i_msg);
+  NA_serverError(const std::string& i_msg = "");
   NA_serverError(void* data, unsigned int len);
   virtual ~NA_serverError();
   std::string actionKey()    { return ActionKey; }
@@ -264,7 +282,7 @@ class NA_serverError : public NetAction {
 
 class NA_frame : public NetAction {
   public:
-  NA_frame(SerializedBikeState* i_state);
+  NA_frame(SerializedBikeState* i_state = NULL);
   NA_frame(void* data, unsigned int len);
   virtual ~NA_frame();
   std::string actionKey()    { return ActionKey; }
@@ -282,7 +300,7 @@ class NA_frame : public NetAction {
 
 class NA_changeName : public NetAction {
   public:
-  NA_changeName(const std::string& i_name);
+  NA_changeName(const std::string& i_name = "");
   NA_changeName(void* data, unsigned int len);
   virtual ~NA_changeName();
   std::string actionKey()    { return ActionKey; }
@@ -300,7 +318,7 @@ class NA_changeName : public NetAction {
 
 class NA_clientsNumber : public NetAction {
   public:
-  NA_clientsNumber(int i_number);
+  NA_clientsNumber(int i_ntcp = 0, int i_nudp = 0, int i_nghosts = 0, int i_nslaves = 0);
   NA_clientsNumber(void* data, unsigned int len);
   virtual ~NA_clientsNumber();
   std::string actionKey()    { return ActionKey; }
@@ -310,10 +328,16 @@ class NA_clientsNumber : public NetAction {
 
   void send(TCPsocket* i_tcpsd, UDPsocket* i_udpsd, UDPpacket* i_sendPacket, IPaddress* i_udpRemoteIP);
 
-  int getNumber();
+  int getNumberTCP() const;
+  int getNumberUDP() const;
+  int getNumberGhosts() const;
+  int getNumberSlaves() const;
 
   private:
-  int m_number;
+  int m_ntcp;
+  int m_nudp;
+  int m_nghosts;
+  int m_nslaves;
 };
 
 // query the number of clients for the munin plugin
@@ -332,7 +356,7 @@ class NA_clientsNumberQuery : public NetAction {
 
 class NA_playingLevel : public NetAction {
   public:
-  NA_playingLevel(const std::string& i_levelId);
+  NA_playingLevel(const std::string& i_levelId = "");
   NA_playingLevel(void* data, unsigned int len);
   virtual ~NA_playingLevel();
   std::string actionKey()    { return ActionKey; }
@@ -373,7 +397,7 @@ class NA_changeClients : public NetAction {
 class NA_playerControl : public NetAction {
   public:
   NA_playerControl(PlayerControl i_control, float i_value);
-  NA_playerControl(PlayerControl i_control, bool i_value);
+  NA_playerControl(PlayerControl i_control = PC_CHANGEDIR, bool i_value = true);
   NA_playerControl(void* data, unsigned int len);
   virtual ~NA_playerControl();
   std::string actionKey()    { return ActionKey; }
@@ -394,7 +418,7 @@ class NA_playerControl : public NetAction {
 
 class NA_clientMode : public NetAction {
   public:
-  NA_clientMode(NetClientMode i_mode);
+  NA_clientMode(NetClientMode i_mode = NETCLIENT_GHOST_MODE);
   NA_clientMode(void* data, unsigned int len);
   virtual ~NA_clientMode();
   std::string actionKey()    { return ActionKey; }
@@ -412,6 +436,7 @@ class NA_clientMode : public NetAction {
 
 class NA_prepareToPlay : public NetAction {
   public:
+  NA_prepareToPlay() : NetAction(false) {}
   NA_prepareToPlay(const std::string& i_id_level, std::vector<int>& i_players);
   NA_prepareToPlay(void* data, unsigned int len);
   virtual ~NA_prepareToPlay();
@@ -432,7 +457,7 @@ class NA_prepareToPlay : public NetAction {
 
 class NA_killAlert : public NetAction {
   public:
-  NA_killAlert(int i_time);
+  NA_killAlert(int i_time = 0);
   NA_killAlert(void* data, unsigned int len);
   virtual ~NA_killAlert();
   std::string actionKey()    { return ActionKey; }
@@ -450,7 +475,7 @@ class NA_killAlert : public NetAction {
 
 class NA_prepareToGo : public NetAction {
   public:
-  NA_prepareToGo(int i_time);
+  NA_prepareToGo(int i_time = 0);
   NA_prepareToGo(void* data, unsigned int len);
   virtual ~NA_prepareToGo();
   std::string actionKey()    { return ActionKey; }
@@ -468,7 +493,7 @@ class NA_prepareToGo : public NetAction {
 
 class NA_gameEvents : public NetAction {
   public:
-  NA_gameEvents(DBuffer* i_buffer);
+  NA_gameEvents(DBuffer* i_buffer = NULL);
   NA_gameEvents(void* data, unsigned int len);
   virtual ~NA_gameEvents();
   std::string actionKey()    { return ActionKey; }
@@ -488,7 +513,7 @@ class NA_gameEvents : public NetAction {
 
 class NA_srvCmd : public NetAction {
   public:
-  NA_srvCmd(const std::string& i_cmd);
+  NA_srvCmd(const std::string& i_cmd = "");
   NA_srvCmd(void* data, unsigned int len);
   virtual ~NA_srvCmd();
   std::string actionKey()    { return ActionKey; }
@@ -506,7 +531,7 @@ class NA_srvCmd : public NetAction {
 
 class NA_srvCmdAsw : public NetAction {
   public:
-  NA_srvCmdAsw(const std::string& i_answer);
+  NA_srvCmdAsw(const std::string& i_answer = "");
   NA_srvCmdAsw(void* data, unsigned int len);
   virtual ~NA_srvCmdAsw();
   std::string actionKey()    { return ActionKey; }
@@ -522,4 +547,77 @@ class NA_srvCmdAsw : public NetAction {
   std::string m_answer;
 };
 
+class NA_slaveClientsPoints : public NetAction {
+  public:
+  NA_slaveClientsPoints();
+  NA_slaveClientsPoints(void* data, unsigned int len);
+  virtual ~NA_slaveClientsPoints();
+  std::string actionKey()    { return ActionKey; }
+  NetActionType actionType() { return NAType; }
+  static std::string ActionKey;
+  static NetActionType NAType;
+
+  void send(TCPsocket* i_tcpsd, UDPsocket* i_udpsd, UDPpacket* i_sendPacket, IPaddress* i_udpRemoteIP);
+
+  const std::vector<NetPointsClient>& getPointsClients() const;
+  void add(NetPointsClient* i_npc);
+
+  private:
+  std::vector<NetPointsClient> m_netPointsClients;
+};
+
+class NA_ping : public NetAction {
+  public:
+  NA_ping(NA_ping* i_ping = NULL); // NULL if that's a ping, the ping for a pong
+  NA_ping(void* data, unsigned int len);
+  virtual ~NA_ping();
+  std::string actionKey()    { return ActionKey; }
+  NetActionType actionType() { return NAType; }
+  static std::string ActionKey;
+  static NetActionType NAType;
+
+  void send(TCPsocket* i_tcpsd, UDPsocket* i_udpsd, UDPpacket* i_sendPacket, IPaddress* i_udpRemoteIP);
+
+  int id() const;
+  bool isPong() const;
+  
+  private:
+  static int m_currentId; // shared id
+
+  int  m_id;   // unique identifier
+  bool m_isPong; // is it an answer of a ping ?
+
+};
+
+/* structure to avoid allocation of the NetAction while netaction are read and manage one by one */
+struct NetActionU {
+public:
+  NetActionU() {}
+
+  NetAction*            master;
+  NA_clientInfos       	clientInfos;
+  NA_udpBindQuery      	udpBindQuery;
+  NA_udpBind           	udpBind;
+  NA_udpBindValidation 	udpBindValidation;
+  NA_chatMessage       	chatMessage;
+  NA_chatMessagePP     	chatMessagePP;
+  NA_serverError       	serverError;
+  NA_frame             	frame;
+  NA_changeName    	changeName;
+  NA_clientsNumber 	clientsNumber;
+  NA_clientsNumberQuery clientsNumberQuery;
+  NA_playingLevel  	playingLevel;
+  NA_changeClients 	changeClients;
+  NA_slaveClientsPoints slaveClientsPoints;
+  NA_playerControl 	playerControl;
+  NA_clientMode    	clientMode;
+  NA_prepareToPlay 	prepareToPlay;
+  NA_prepareToGo   	prepareToGo;
+  NA_killAlert     	killAlert;
+  NA_gameEvents    	gameEvents;
+  NA_srvCmd    	   	srvCmd;
+  NA_srvCmdAsw 	   	srvCmdAsw;
+  NA_ping      	   	ping;
+};
+
 #endif
